"""
Test for adding packages
"""

import pytest

from copr.v3 import Client
from copr.test import config_location, mock


@pytest.mark.parametrize('method_name', ['add', 'edit'])
@mock.patch("copr.v3.proxies.Request.send")
def test_package_distgit(request, method_name):
    mock_client = Client.create_from_config_file(config_location)
    method = getattr(mock_client.package_proxy, method_name)
    method("fooUser", "test", "mock", "distgit",
           {"committish": "master", "distgit": "fedora"})
    assert len(request.call_args_list) == 1
    call = request.call_args_list[0]
    endpoint = call[1]["endpoint"]
    args = call[1]
    assert args['method'] == 'POST'
    base_url = "/package/{0}".format(method_name)
    assert endpoint == base_url + "/{ownername}/{projectname}/{package_name}/{source_type_text}"
    params = args['params']
    assert params == {'ownername': 'fooUser', 'projectname': 'test',
                      'package_name': 'mock', 'source_type_text': 'distgit'}
    assert args['data'] == {'committish': 'master', 'distgit': 'fedora',
                            'package_name': 'mock'}



from munch import Munch


class CoprException(Exception):
    """
    Base Copr exception
    """
    def __init__(self, msg=None, response=None):
        super(CoprException, self).__init__(msg)
        msg = msg or "Unspecified error"
        self.result = Munch(error=msg, __response__=response)


class CoprRequestException(CoprException):
    """
    Raised when the API request doesn't proceed successfully
    """
    def __str__(self):
        errors = self.result.error.split("\n")

        # A list of errors signalizes a form validation error
        # If there is only one error in the list, just return its value
        if len(errors) == 1:
            return str(errors[0])

        # Show one error per line
        result = ""
        for error in errors:
            result += "\n- {0}".format(error)
        return result


class CoprNoResultException(CoprException):
    """
    Raised when no result data is returned
    """
    pass


class CoprTimeoutException(CoprException):
    """
    Raised when the API request timeouted
    """
    pass


class CoprValidationException(CoprException):
    """
    Raised when the data sent from client to API are not valid
    """
    pass


class CoprNoConfigException(CoprException):
    """
    Exception thrown when no config file is found
    We left this exception in our code because someone can still catch it
    """
    pass


class CoprConfigException(CoprException):
    """
    Exception thrown when the config file is incomplete or malformed.
    """
    pass


class CoprAuthException(CoprException):
    """
    Copr authentication failure
    """



import os
import pytest
import tempfile
import shutil
from json import loads
from copr.test import mock
from copr.test.client_v3.test_auth import FakeCache
from copr.v3 import ModuleProxy


class TestModuleProxy(object):
    config_auth = {
        "copr_url": "http://copr",
        "login": "test_api_login",
        "token": "test_api_token",
    }

    def setup_method(self, method):
        self.tmpdir = tempfile.mkdtemp(prefix='test-python-copr-modules')
        self.yaml_file = os.path.join(self.tmpdir, 'test.yaml')
        with open(self.yaml_file, 'w') as f:
            f.write("")

    def teardown_method(self, method):
        shutil.rmtree(self.tmpdir)

    @pytest.mark.parametrize('distgit_opt', [None, 'fedora'])
    @mock.patch('copr.v3.requests.requests.Session.request')
    def test_module_dist_git_choice_url(self, request, distgit_opt):
        proxy = ModuleProxy(self.config_auth)
        proxy.auth.cache = FakeCache(None)
        proxy.build_from_url('owner', 'project', 'http://test.yaml',
                             distgit=distgit_opt)

        assert len(request.call_args_list) == 1
        call = request.call_args_list[0]
        kwargs = call[1]
        json = kwargs['json']
        if distgit_opt is None:
            assert 'distgit' not in json
        else:
            assert json['distgit'] == distgit_opt

        assert json['scmurl'] == 'http://test.yaml'

    @pytest.mark.parametrize('distgit_opt', [None, 'fedora'])
    @mock.patch('copr.v3.requests.requests.Session.request')
    def test_module_dist_git_choice_upload(self, request, distgit_opt):
        proxy = ModuleProxy(self.config_auth)
        proxy.auth.cache = FakeCache(None)
        proxy.build_from_file('owner', 'project',
                              self.yaml_file,
                              distgit=distgit_opt)

        assert len(request.call_args_list) == 1
        call = request.call_args_list[0]
        kwargs = call[1]
        json = kwargs['json']

        assert json is None

        # ('json', 'jsonstr', 'application/json')
        json_encoded = kwargs['data'].encoder.fields['json']
        json = loads(json_encoded[1])

        if distgit_opt is None:
            assert json is None
        else:
            assert json['distgit'] == distgit_opt



from __future__ import absolute_import

import warnings

from . import BaseProxy
from ..requests import munchify, POST, GET, PUT
from ..helpers import for_all_methods, bind_proxy


def _compat_use_bootstrap_container(data, value):
    if value is None:
        return
    data["bootstrap"] = "on" if value else "off"
    warnings.warn("The 'use_bootstrap_container' argument is obsoleted by "
                  "'bootstrap' and 'bootstrap_image'")

@for_all_methods(bind_proxy)
class ProjectProxy(BaseProxy):

    def get(self, ownername, projectname):
        """
        Return a project

        :param str ownername:
        :param str projectname:
        :return: Munch
        """
        endpoint = "/project"
        params = {
            "ownername": ownername,
            "projectname": projectname,
        }
        response = self.request.send(endpoint=endpoint, params=params)
        return munchify(response)

    def get_list(self, ownername=None, pagination=None):
        """
        Return a list of projects

        :param str ownername:
        :param pagination:
        :return: Munch
        """
        endpoint = "/project/list"
        params = {
            "ownername": ownername,
        }
        params.update(pagination or {})
        response = self.request.send(endpoint=endpoint, params=params)
        return munchify(response)

    def search(self, query, pagination=None):
        """
        Return a list of projects based on fulltext search

        :param str query:
        :param pagination:
        :return: Munch
        """
        endpoint = "/project/search"
        params = {
            "query": query,
        }
        params.update(pagination or {})
        response = self.request.send(endpoint=endpoint, params=params)
        return munchify(response)

    def add(self, ownername, projectname, chroots, description=None, instructions=None, homepage=None,
            contact=None, additional_repos=None, unlisted_on_hp=False, enable_net=False, persistent=False,
            auto_prune=True, use_bootstrap_container=None, devel_mode=False,
            delete_after_days=None, multilib=False, module_hotfixes=False,
            bootstrap=None, bootstrap_image=None, isolation=None,
            fedora_review=None, appstream=True, runtime_dependencies=None):
        """
        Create a project

        :param str ownername:
        :param str projectname:
        :param list chroots:
        :param str description:
        :param str instructions:
        :param str homepage:
        :param str contact:
        :param list additional_repos:
        :param bool unlisted_on_hp: project will not be shown on Copr homepage
        :param bool enable_net: if builder can access net for builds in this project
        :param bool persistent: if builds and the project are undeletable
        :param bool auto_prune: if backend auto-deletion script should be run for the project
        :param bool use_bootstrap_container: obsoleted, use the 'bootstrap'
            argument and/or the 'bootstrap_image'.
        :param bool devel_mode: if createrepo should run automatically
        :param int delete_after_days: delete the project after the specfied period of time
        :param bool module_hotfixes: make packages from this project available
                                     on along with packages from the active module streams.
        :param str bootstrap: Mock bootstrap feature setup.
            Possible values are 'default', 'on', 'off', 'image'.
        :param str bootstrap_image: Name of the container image to initialize
            the bootstrap chroot from.  This also implies 'bootstrap=image'.
            This is a noop parameter and its value is ignored.
        :param str isolation: Mock isolation feature setup.
            Possible values are 'default', 'simple', 'nspawn'.
        :param bool fedora_review: Run fedora-review tool for packages
                                   in this project
        :param bool appstream: Disable or enable generating the appstream metadata
        :param string runtime_dependencies: List of external repositories
            (== dependencies, specified as baseurls) that will be automatically
            enabled together with this project repository.
        :return: Munch
        """
        endpoint = "/project/add/{ownername}"
        params = {
            "ownername": ownername,
        }
        data = {
            "name": projectname,
            "chroots": chroots,
            "description": description,
            "instructions": instructions,
            "homepage": homepage,
            "contact": contact,
            "additional_repos": additional_repos,
            "unlisted_on_hp": unlisted_on_hp,
            "enable_net": enable_net,
            "persistent": persistent,
            "auto_prune": auto_prune,
            "bootstrap": bootstrap,
            "bootstrap_image": bootstrap_image,
            "isolation": isolation,
            "devel_mode": devel_mode,
            "delete_after_days": delete_after_days,
            "multilib": multilib,
            "module_hotfixes": module_hotfixes,
            "fedora_review": fedora_review,
            "appstream": appstream,
            "runtime_dependencies": runtime_dependencies,
        }

        _compat_use_bootstrap_container(data, use_bootstrap_container)

        response = self.request.send(
            endpoint=endpoint,
            method=POST,
            params=params,
            data=data,
            auth=self.auth,
        )
        return munchify(response)

    def edit(self, ownername, projectname, chroots=None, description=None, instructions=None, homepage=None,
             contact=None, additional_repos=None, unlisted_on_hp=None, enable_net=None,
             auto_prune=None, use_bootstrap_container=None, devel_mode=None,
             delete_after_days=None, multilib=None, module_hotfixes=None,
             bootstrap=None, bootstrap_image=None, isolation=None,
             fedora_review=None, appstream=None, runtime_dependencies=None):
        """
        Edit a project

        :param str ownername:
        :param str projectname:
        :param list chroots:
        :param str description:
        :param str instructions:
        :param str homepage:
        :param str contact:
        :param list repos:
        :param bool unlisted_on_hp: project will not be shown on Copr homepage
        :param bool enable_net: if builder can access net for builds in this project
        :param bool auto_prune: if backend auto-deletion script should be run for the project
        :param bool use_bootstrap_container: obsoleted, use the 'bootstrap'
            argument and/or the 'bootstrap_image'.
        :param bool devel_mode: if createrepo should run automatically
        :param int delete_after_days: delete the project after the specfied period of time
        :param bool module_hotfixes: make packages from this project available
                                     on along with packages from the active module streams.
        :param str bootstrap: Mock bootstrap feature setup.
            Possible values are 'default', 'on', 'off', 'image'.
        :param str isolation: Mock isolation feature setup.
            Possible values are 'default', 'simple', 'nspawn'.
        :param str bootstrap_image: Name of the container image to initialize
            the bootstrap chroot from.  This also implies 'bootstrap=image'.
            This is a noop parameter and its value is ignored.
        :param bool fedora_review: Run fedora-review tool for packages
                                   in this project
        :param bool appstream: Disable or enable generating the appstream metadata
        :return: Munch
        """
        endpoint = "/project/edit/{ownername}/{projectname}"
        params = {
            "ownername": ownername,
            "projectname": projectname,
        }
        data = {
            "chroots": chroots,
            "description": description,
            "instructions": instructions,
            "homepage": homepage,
            "contact": contact,
            "repos": additional_repos,
            "unlisted_on_hp": unlisted_on_hp,
            "enable_net": enable_net,
            "auto_prune": auto_prune,
            "bootstrap": bootstrap,
            "isolation": isolation,
            "bootstrap_image": bootstrap_image,
            "devel_mode": devel_mode,
            "delete_after_days": delete_after_days,
            "multilib": multilib,
            "module_hotfixes": module_hotfixes,
            "fedora_review": fedora_review,
            "appstream": appstream,
            "runtime_dependencies": runtime_dependencies,
        }

        _compat_use_bootstrap_container(data, use_bootstrap_container)

        response = self.request.send(
            endpoint=endpoint,
            method=POST,
            params=params,
            data=data,
            auth=self.auth,
        )
        return munchify(response)

    def delete(self, ownername, projectname):
        """
        Delete a project

        :param str ownername:
        :param str projectname:
        :return: Munch
        """
        endpoint = "/project/delete/{ownername}/{projectname}"
        params = {
            "ownername": ownername,
            "projectname": projectname,
        }
        data = {
            "verify": True,
        }
        response = self.request.send(
            endpoint=endpoint,
            method=POST,
            params=params,
            data=data,
            auth=self.auth,
        )
        return munchify(response)

    def fork(self, ownername, projectname, dstownername, dstprojectname, confirm=False):
        """
        Fork a project

        :param str ownername: owner of a source project
        :param str projectname: name of a source project
        :param str dstownername: owner of a destination project
        :param str dstprojectname: name of a destination project
        :param bool confirm: if forking into a existing project, this needs to be set to True,
                             to confirm that user is aware of that
        :return: Munch
        """
        endpoint = "/project/fork/{ownername}/{projectname}"
        params = {
            "ownername": ownername,
            "projectname": projectname,
        }
        data = {
            "name": dstprojectname,
            "ownername": dstownername,
            "confirm": confirm,
        }
        response = self.request.send(
            endpoint=endpoint,
            method=POST,
            params=params,
            data=data,
            auth=self.auth,
        )
        return munchify(response)

    def get_permissions(self, ownername, projectname):
        """
        Get project permissions

        :param str ownername: owner of the project
        :param str projectname: name of the project
        :return Munch: a dictionary in format
            ``{username: {permission: state, ... }, ...}`` where ``username``
            identifies an existing copr user, ``permission`` is one of
            ``admin|builder`` and state is one of ``nothing|approved|request``.
        """

        endpoint = "/project/permissions/get/{ownername}/{projectname}/"
        params = {
            "ownername": ownername,
            "projectname": projectname,
        }
        response = self.request.send(
            endpoint=endpoint, params=params, auth=self.auth)
        return munchify(response)

    def set_permissions(self, ownername, projectname, permissions):
        """
        Set (or change) permissions for a project

        :param str ownername: owner of the updated project
        :param str projectname: name of the updated project
        :param dict permissions: the expected format is
            ``{username: {permission: state, ...}, ...}``
            where ``username`` identifies an existing copr user, ``permission``
            is one of ``builder|admin`` and ``state`` value is one of
            ``nothing|request|approved``.  It is OK to set only ``bulider`` or
            only ``admin`` permission; any unspecified ``permission`` is then
            (a) set to ``nothing`` (if the permission entry is newly created),
            or (b) kept unchanged (if an existing permission entry is edited).
            If more than one ``username`` is specified in single
            ``set_permissions()`` request, the approach is to configure
            *all-or-nothing* (any error makes the whole operation fail and
            no-op).
        """

        endpoint = "/project/permissions/set/{ownername}/{projectname}/"
        params = {
            "ownername": ownername,
            "projectname": projectname,
        }
        self.request.send(
            endpoint=endpoint,
            method=PUT,
            params=params,
            data=permissions,
            auth=self.auth,
        )

    def request_permissions(self, ownername, projectname, permissions):
        """
        Request/cancel request/drop your permissions on project

        :param str ownername: owner of the requested project
        :param str projectname: name of the requested project
        :param dict permissions: the desired permissions user wants to have on
            the requested copr project.  The format is
            ``{permission: bool, ...}``, where ``permission`` is one of
            ``builder|admin`` and ``bool`` is
            (a) ``True`` for *requesting* the role or
            (b) ``False`` for *dropping* the role (or for *cancelling* of
            previous request).
        """
        endpoint = "/project/permissions/request/{ownername}/{projectname}/"
        params = {
            "ownername": ownername,
            "projectname": projectname,
        }
        self.request.send(
            endpoint=endpoint,
            method=PUT,
            params=params,
            data=permissions,
            auth=self.auth,
        )

    def regenerate_repos(self, ownername, projectname):
        """
        Regenerate repositories for a project

        :param str ownername: owner of the project to regenerate
        :param str projectname: name of the project to regenerate
        """
        endpoint = "/project/regenerate-repos/{ownername}/{projectname}"
        params = {
            "ownername": ownername,
            "projectname": projectname
        }
        response = self.request.send(
            endpoint=endpoint, method=PUT, params=params, auth=self.auth)
        return munchify(response)



"""
Base authentication interface
"""

import os
import json
import time
import errno
from filelock import FileLock

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse


class BaseAuth:
    """
    Base authentication class
    There is a more standard way of implementing custom authentication classes,
    please see https://docs.python-requests.org/en/latest/user/authentication/
    We can eventually implement it using `requests.auth`.
    """

    def __init__(self, config):
        self.config = config
        self.cache = AuthCache(config)
        self.username = None
        self.expiration = None

        # These attributes will be directly passed to the requests.request(...)
        # calls. Various authentication mechanisms should set them accordingly.
        self.auth = None
        self.cookies = None

    @property
    def expired(self):
        """
        Are the authentication tokens, cookies, etc, expired?

        We know this for example for cached cookies. It can be tricky because
        it can be expired regardless of the expiration time when frontend
        decides to e.g. revoke all tokens, but we get to know only when sending
        a request.

        But when expiration time is gone, we for sure know the cookie is
        expired. That will help us avoid sending requests that we know will be
        unsuccessful.
        """
        if not self.expiration:
            return False
        return self.expiration < time.time()

    def make_expensive(self):
        """
        Perform the authentication process. This is the most expensive part.
        The point is to set `self.username`, and some combination of
        `self.auth` and `self.cookies`.
        """
        raise NotImplementedError

    def make(self, reauth=False):
        """
        Perform the authentication process. It can be expensive, so we ensure
        caching with locks, expiration checks, etc.
        If `reauth=True` is set, then any cached cookies are ignored and the
        authentication is done from scratch
        """
        # If we know an username, the authentication must have been done
        # previously and there is no need to do it again.
        # Unless we specifically request an re-auth
        auth_done = bool(self.username) and not reauth
        if auth_done:
            return

        with self.cache.lock:
            # Try to load session data from cache
            if not reauth and self._load_cache():
                auth_done = True

            # If we don't have any cached cookies or they are expired
            if not auth_done or self.expired:
                self.make_expensive()
                self._save_cache()

    def _load_cache(self):
        session_data = self.cache.load_session()
        if session_data:
            token = session_data["session"]
            self.username = session_data["name"]
            self.cookies = {"session": token}
            self.expiration = session_data["expiration"]
        return bool(session_data)

    def _save_cache(self):
        # For now, we don't want to cache (login, token) information, only
        # session cookies (for e.g. GSSAPI)
        if not self.cookies:
            return

        data = {
            "name": self.username,
            "session": self.cookies["session"],
            "expiration": self.expiration,
        }
        self.cache.save_session(data)


class AuthCache:
    """
    Some authentication methods are expensive and we want to use them only
    for an initial authentication, chache their value, and use until expiration.
    """

    def __init__(self, config):
        self.config = config

    @property
    def session_file(self):
        """
        Path to the cached file for a given Copr instance
        """
        url = urlparse(self.config["copr_url"]).netloc
        cachedir = os.path.join(os.path.expanduser("~"), ".cache", "copr")
        try:
            os.makedirs(cachedir)
        except OSError as err:
            if err.errno != errno.EEXIST:
                raise
        return os.path.join(cachedir, url + "-session")

    @property
    def lock_file(self):
        """
        Path to the lock file for a given Copr instance
        """
        return self.session_file + ".lock"

    @property
    def lock(self):
        """
        Allow the user to do `with cache.lock:`
        """
        return FileLock(self.lock_file)

    def load_session(self):
        """
        Load the session data from a cache file
        """
        if not os.path.exists(self.session_file):
            return None
        with open(self.session_file, "r") as fp:
            return json.load(fp)

    def save_session(self, session_data):
        """
        Save the session data to a cache file
        """
        with open(self.session_file, "w") as file:
            session_data["expiration"] = time.time() + 10 * 3600  # +10 hours
            file.write(json.dumps(session_data, indent=4) + "\n")



from requests import Response
from copr.v3 import Client, BuildProxy
from copr.v3.requests import Request

from copr.test import config_location, mock


@mock.patch.object(Request, "send")
class TestBuildProxy(object):
    config = {"copr_url": "http://copr", "login": "test", "token": "test"}

    def test_get(self, send):
        response = mock.Mock(spec=Response)
        response.json.return_value = {"id": 1, "foo": "bar"}
        send.return_value = response

        build_proxy = BuildProxy(self.config)
        build = build_proxy.get(1)
        assert build.id == 1
        assert build.foo == "bar"


@mock.patch('copr.v3.proxies.Request.send')
def test_build_distgit(send):
    mock_client = Client.create_from_config_file(config_location)
    mock_client.build_proxy.create_from_distgit(
        "praiskup", "ping", "mock", committish="master",
    )
    assert len(send.call_args_list) == 1
    call = send.call_args_list[0]
    args = call[1]
    assert args['method'] == 'POST'
    assert args['endpoint'] == '/build/create/distgit'
    assert args['data'] == {
        'ownername': 'praiskup', 'projectname': 'ping',
        'distgit': None, 'namespace': None, 'package_name': 'mock',
        'committish': 'master', 'project_dirname': None}



from __future__ import absolute_import

import os
from . import BaseProxy
from ..requests import FileRequest, munchify, POST
from ..helpers import for_all_methods, bind_proxy


@for_all_methods(bind_proxy)
class ProjectChrootProxy(BaseProxy):

    def get(self, ownername, projectname, chrootname):
        """
        Return a configuration of a chroot in a project

        :param str ownername:
        :param str projectname:
        :param str chrootname:
        :return: Munch
        """
        endpoint = "/project-chroot"
        params = {
            "ownername": ownername,
            "projectname": projectname,
            "chrootname": chrootname,
        }
        response = self.request.send(endpoint=endpoint, params=params)
        return munchify(response)

    def get_build_config(self, ownername, projectname, chrootname):
        """
        Return a build configuration of a chroot in a project

        :param str ownername:
        :param str projectname:
        :param str chrootname:
        :return: Munch
        """
        endpoint = "/project-chroot/build-config"
        params = {
            "ownername": ownername,
            "projectname": projectname,
            "chrootname": chrootname,
        }
        response = self.request.send(endpoint=endpoint, params=params)
        return munchify(response)

    # pylint: disable=too-many-arguments
    def edit(self, ownername, projectname, chrootname, additional_packages=None, additional_repos=None,
             additional_modules=None, comps=None, delete_comps=False, with_opts=None, without_opts=None,
             bootstrap=None, bootstrap_image=None, isolation=None,
             reset_fields=None):
        """
        Edit a chroot configuration in a project

        :param str ownername:
        :param str projectname:
        :param str chrootname:
        :param list additional_packages: buildroot packages for the chroot
        :param list additional_repos: buildroot additional additional_repos
        :param list additional_modules: additional modules for the chroot
        :param str comps: file path to the comps.xml file
        :param bool delete_comps: if True, current comps.xml will be removed
        :param list with_opts: Mock --with option
        :param list without_opts: Mock --without option
        :param str bootstrap: Allowed values 'on', 'off', 'image', 'default',
                              'untouched' (equivalent to None)
        :param str bootstrap_image: Implies 'bootstrap=image'.
        :param str isolation: Mock isolation feature setup.
            Possible values are 'default', 'simple', 'nspawn'.
        :param list reset_fields: list of chroot attributes, that should be
            reseted to their respective defaults. Possible values are
            `additional_packages`, `additional_modules`, `isolation`, etc. See
            the output of `ProjectProxy.get` for all the possible field names.
        :return: Munch
        """
        endpoint = "/project-chroot/edit/{ownername}/{projectname}/{chrootname}"
        params = {
            "ownername": ownername,
            "projectname": projectname,
            "chrootname": chrootname,
        }

        if bootstrap_image:
            bootstrap = 'custom_image'

        data = {
            "additional_repos": additional_repos,
            "additional_packages": additional_packages,
            "additional_modules": additional_modules,
            "delete_comps": delete_comps,
            "with_opts": with_opts,
            "without_opts": without_opts,
            "bootstrap": bootstrap,
            "bootstrap_image": bootstrap_image,
            "isolation": isolation,
            "reset_fields": reset_fields,
        }
        files = {}
        if comps:
            comps_f = open(comps, "rb")
            files["upload_comps"] = (os.path.basename(comps_f.name), comps_f, "application/text")

        request = FileRequest(
            api_base_url=self.api_base_url,
            files=files,
            connection_attempts=self.config.get("connection_attempts", 1)
        )
        response = request.send(
            endpoint=endpoint,
            method=POST,
            params=params,
            data=data,
            auth=self.auth,
        )
        return munchify(response)



from __future__ import absolute_import

from . import BaseProxy
from ..requests import munchify
from ..helpers import for_all_methods, bind_proxy


@for_all_methods(bind_proxy)
class BuildChrootProxy(BaseProxy):
    def get(self, build_id, chrootname):
        """
        Return a build chroot

        :param int build_id:
        :param str chrootname:
        :return: Munch
        """
        endpoint = "/build-chroot"
        params = {
            "build_id": build_id,
            "chrootname": chrootname,
        }
        response = self.request.send(endpoint=endpoint, params=params)
        return munchify(response)

    def get_list(self, build_id, pagination=None):
        """
        Return a list of build chroots

        :param int build_id:
        :param str chrootname:
        :param pagination:
        :return: Munch
        """
        endpoint = "/build-chroot/list"
        params = {
            "build_id": build_id,
        }
        params.update(pagination or {})
        response = self.request.send(endpoint=endpoint, params=params)
        return munchify(response)

    def get_build_config(self, build_id, chrootname):
        """
        Return a build config for a build chroot

        :param int build_id:
        :param str chrootname:
        :return: Munch
        """
        endpoint = "/build-chroot/build-config"
        params = {
            "build_id": build_id,
            "chrootname": chrootname,
        }
        response = self.request.send(endpoint=endpoint, params=params)
        return munchify(response)

    def get_built_packages(self, build_id, chrootname):
        """
        Return built packages (NEVRA dicts) for a given build chroot

        :param int build_id:
        :param str chrootname:
        :return: Munch
        """
        endpoint = "/build-chroot/built-packages"
        params = {
            "build_id": build_id,
            "chrootname": chrootname,
        }
        response = self.request.send(endpoint=endpoint, params=params)
        return munchify(response)



"""
Authentication via (login, token)
"""

from copr.v3.auth.base import BaseAuth


class ApiToken(BaseAuth):
    """
    The standard authentication via `(login, token)` from `~/.config/copr`
    """
    def make_expensive(self):
        self.auth = self.config["login"], self.config["token"]
        self.username = self.config.get("username")



from copr.v3.proxies import BaseProxy


class TestBaseProxy(object):
    def test_api_base_url(self):
        proxy = BaseProxy({"copr_url": "http://copr"})
        assert proxy.api_base_url == "http://copr/api_3/"

        # Slashes or port number should not be a problem
        proxy = BaseProxy({"copr_url": "http://copr:5000/"})
        assert proxy.api_base_url == "http://copr:5000/api_3/"



from __future__ import absolute_import

from .helpers import config_from_file
from .client import Client
from .proxies import BaseProxy
from .proxies.project import ProjectProxy
from .proxies.build import BuildProxy
from .proxies.package import PackageProxy
from .proxies.mock_chroot import MockChrootProxy
from .proxies.project_chroot import ProjectChrootProxy
from .proxies.build_chroot import BuildChrootProxy
from .proxies.module import ModuleProxy
from .exceptions import (CoprException,
                         CoprRequestException,
                         CoprNoResultException,
                         CoprValidationException,
                         CoprNoConfigException,
                         CoprConfigException,
                         CoprAuthException)


__all__ = [
    "config_from_file",
    "Client",

    "BaseProxy",
    "BuildProxy",
    "PackageProxy",
    "MockChrootProxy",
    "ProjectChrootProxy",
    "BuildChrootProxy",
    "ModuleProxy",

    "CoprException",
    "CoprRequestException",
    "CoprNoResultException",
    "CoprValidationException",
    "CoprNoConfigException",
    "CoprConfigException",
    "CoprAuthException",
]
